//+------------------------------------------------------------------+
//|                     Deus EA                                      |
//|     Copyright 2024, MetaQuotes Ltd.                              |
//|                    https://www.mql5.com                          |
//+------------------------------------------------------------------+
#include <Trade\Trade.mqh>
CTrade trade;

//--- Input parameters
input double   InpLots           = 0.1;   // Lot size
input double   InpStopLoss       = 50;    // Stop loss in points
input double   InpTakeProfit     = 50;    // Take profit in points
input double   InpTrailingStop   = 25;    // Trailing stop in points

//--- RSI parameters
input int      InpRSI_Period     = 7;     // RSI period
input double   InpRSI_Overbought = 35.0;  // RSI overbought level
input double   InpRSI_Oversold   = 15.0;  // RSI oversold level

//--- Moving Average parameters
input int                  InpMA_Period   = 25;          // Moving Average period
input ENUM_MA_METHOD       InpMA_Method   = MODE_SMA;    // Moving Average method
input ENUM_APPLIED_PRICE   InpMA_Price    = PRICE_CLOSE; // Applied price for MA

//--- Variables
int    ExtHandleMA;  // MA handle
int    ExtHandleRSI; // RSI handle

//--- Function prototypes
void ApplyTrailingStop();
void OpenPosition(const ENUM_POSITION_TYPE position_type);     // pass position_type directly
void ClosePositions(const ENUM_POSITION_TYPE position_type);   // pass position_type directly

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   ExtHandleMA=iMA(_Symbol, _Period, InpMA_Period, 0, InpMA_Method, InpMA_Price);
   if(ExtHandleMA==INVALID_HANDLE)
     {
      PrintFormat("The iMA(%d) object was not created. Error %d", InpMA_Period, GetLastError());
      return(INIT_FAILED);
     }
   ExtHandleRSI=iRSI(_Symbol, _Period, InpRSI_Period, PRICE_CLOSE);
   if(ExtHandleRSI==INVALID_HANDLE)
     {
      PrintFormat("The iRSI(%d) object was not created. Error %d", InpRSI_Period, GetLastError());
      return(INIT_FAILED);
     }
   Print("Deus EA initialized successfully.");
   return(INIT_SUCCEEDED);
  }

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   Print("Deus EA deinitialized. Reason: ", reason);
  }

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   double RsiValue= 0;  // RSI value
   double MaValue = 0;  // Moving Average value
   double Close   = 0;  // Price Close
   
//--- Get RSI value
   RsiValue = GetRSI(0);
   if(RsiValue == EMPTY_VALUE)
     {
      Print("Error calculating RSI");
      return;
     }

//--- Get Moving Average value
   MaValue = GetMA(0);
   if(MaValue == EMPTY_VALUE)
     {
      Print("Error calculating Moving Average");
      return;
     }

//--- Get Close price value
   Close = GetClose(1);
   if(Close == 0)
     {
      Print("Error getting price Close");
      return;
     }

//--- Check for Buy Signal
   if(RsiValue < InpRSI_Oversold && Close > MaValue)
     {
      if(PositionsTotal() == 0)
        {
         ClosePositions(POSITION_TYPE_SELL);
         OpenPosition(POSITION_TYPE_BUY);
        }
     }

//--- Check for Sell Signal
   if(RsiValue > InpRSI_Overbought && Close < MaValue)
     {
      if(PositionsTotal() == 0)
        {
         ClosePositions(POSITION_TYPE_BUY);
         OpenPosition(POSITION_TYPE_SELL);
        }
     }

//--- Apply trailing stop if specified
   if(InpTrailingStop > 0)
     {
      ApplyTrailingStop();
     }
  }

//+------------------------------------------------------------------+
//| Function to open a position                                      |
//+------------------------------------------------------------------+
void OpenPosition(const ENUM_POSITION_TYPE position_type)
  {
//---
   MqlTick tick={};
   ResetLastError();
   if(!SymbolInfoTick(_Symbol, tick))
     {
      PrintFormat("%s: SymbolInfoTick() failed. Error code: %d",__FUNCTION__, GetLastError());
      return;
     }
//--- Determine price stop loss, and take profit levels
   double price= (position_type == POSITION_TYPE_BUY) ? tick.ask : tick.bid;
   double sl   = (position_type == POSITION_TYPE_BUY) ? price - InpStopLoss * _Point : price + InpStopLoss * _Point;
   double tp   = (position_type == POSITION_TYPE_BUY) ? price + InpTakeProfit * _Point : price - InpTakeProfit * _Point;

   ENUM_ORDER_TYPE order_type=(position_type==POSITION_TYPE_BUY ? ORDER_TYPE_BUY : ORDER_TYPE_SELL);
   bool result = trade.PositionOpen(_Symbol, order_type, InpLots, price, sl, tp, "Zeus EA");

   if(result)
     {
      Print("Position opened successfully. Type: ", EnumToString(position_type), ", Price: ", DoubleToString(price, _Digits));
     }
   else
     {
      Print("Failed to open position. Error code: ", GetLastError());
     }
  }
//+------------------------------------------------------------------+
//| Function to close positions                                      |
//+------------------------------------------------------------------+
void ClosePositions(const ENUM_POSITION_TYPE position_type)
  {
   for(int i = PositionsTotal() - 1; i >= 0; i--)
     {
      ulong ticket = PositionGetTicket(i);
      if(ticket==0)
         continue;
      //--- Check if the positions type matches the order type to be closed
      if(PositionGetInteger(POSITION_TYPE) == position_type)
        {
         if(!trade.PositionClose(ticket))
           {
            Print("Failed to close position. Error code: ", GetLastError());
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| Function to apply trailing stop                                  |
//+------------------------------------------------------------------+
void ApplyTrailingStop()
  {
   MqlTick tick={};
   ResetLastError();
   if(!SymbolInfoTick(_Symbol, tick))
     {
      PrintFormat("%s: SymbolInfoTick() failed. Error code: %d",__FUNCTION__, GetLastError());
      return;
     }
   for(int i = PositionsTotal() - 1; i >= 0; i--)
     {
      if(PositionGetSymbol(i) == _Symbol)
        {
         double price= (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY) ? tick.bid : tick.ask;
         double sl   = (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY) ? price - InpTrailingStop * _Point : price + InpTrailingStop * _Point;

         //--- Trailing  stop logic for buy positions
         if(PositionGetInteger(POSITION_TYPE) == ORDER_TYPE_BUY && PositionGetDouble(POSITION_SL) < sl)
           {
            if(!trade.PositionModify(PositionGetInteger(POSITION_TICKET), sl, PositionGetDouble(POSITION_TP)))
              {
               Print("Failed to modify position. Error code: ", GetLastError());
              }
           }

         //--- Trailing stop logic for sell positions
         else
            if(PositionGetInteger(POSITION_TYPE) == ORDER_TYPE_SELL && PositionGetDouble(POSITION_SL) > sl)
              {
               if(!trade.PositionModify(PositionGetInteger(POSITION_TICKET), sl, PositionGetDouble(POSITION_TP)))
                 {
                  Print("Failed to modify position. Error code: ", GetLastError());
                 }
              }
        }
     }
  }
//+------------------------------------------------------------------+
//| Function returns the price Close                                 |
//+------------------------------------------------------------------+
double GetClose(const int index)
  {
   double array[1];
   ResetLastError();
   if(CopyClose(_Symbol, _Period, index, 1, array)!=1)
     {
      PrintFormat("%s: CopyClose() failed. Error %d",__FUNCTION__, GetLastError());
      return(0);
     }
   return(array[0]);
  }
//+------------------------------------------------------------------+
//| Function returns RSI indicator value                             |
//+------------------------------------------------------------------+
double GetRSI(const int index)
  {
   double array[1];
   ResetLastError();
   if(CopyBuffer(ExtHandleRSI, 0, index, 1, array)!=1)
     {
      PrintFormat("%s: CopyBuffer() failed. Error %d",__FUNCTION__, GetLastError());
      return(EMPTY_VALUE);
     }
   return(array[0]);
  }
//+------------------------------------------------------------------+
//| Function returns MA indicator value                              |
//+------------------------------------------------------------------+
double GetMA(const int index)
  {
   double array[1];
   ResetLastError();
   if(CopyBuffer(ExtHandleMA, 0, index, 1, array)!=1)
     {
      PrintFormat("%s: CopyBuffer() failed. Error %d",__FUNCTION__, GetLastError());
      return(EMPTY_VALUE);
     }
   return(array[0]);
  }
//+------------------------------------------------------------------+
